import React, { useMemo } from "react";
import { Page } from "@/pages/base";
import { Button, Container, Stepper, StepProps, Theme, Typography } from "@material-ui/core";
import { Link as RouterLink } from "react-router-dom";
import { route } from "@/routing";
import { useTranslation } from "react-i18next";
import { useSelector } from "react-redux";
import { AppState } from "@/state/reducer";
import { getMissingStudentData, Student } from "@/data";
import { Deadlines, Edition, getEditionDeadlines } from "@/data/edition";
import { Description as DescriptionIcon } from "@material-ui/icons"
import { Actions, Step } from "@/components";
import { getInternshipProposalStatus, InternshipProposalState, InternshipProposalStatus } from "@/state/reducer/proposal";
import { createStyles, makeStyles } from "@material-ui/core/styles";


const getColorByStatus = (status: InternshipProposalStatus, theme: Theme) => {
    switch (status) {
        case "awaiting":
            return theme.palette.info.dark;
        case "accepted":
            return theme.palette.success.dark;
        case "declined":
            return theme.palette.error.dark;
        case "draft":
            return theme.palette.grey["800"];
        default:
            return "textPrimary";
    }
}

const useStatusStyles = makeStyles((theme: Theme) => {
    const colorByStatusGetter = ({ status }: any) => getColorByStatus(status, theme);

    return createStyles({
        foreground: {
            color: colorByStatusGetter
        },
        background: {
            backgroundColor: colorByStatusGetter
        }
    })
})

const ProposalStatus = () => {
    const status = useSelector<AppState, InternshipProposalStatus>(state => getInternshipProposalStatus(state.proposal))
    const classes = useStatusStyles({ status });

    const { t } = useTranslation();

    return <span className={ classes.foreground }>{ t(`proposal.status.${status}`) }</span>;
}

const ProposalStep = (props: StepProps) => {
    const { t } = useTranslation();

    const { sent } = useSelector<AppState, InternshipProposalState>(state => state.proposal);
    const deadlines = useSelector<AppState, Deadlines>(state => getEditionDeadlines(state.edition as Edition)); // edition cannot be null at this point

    return <Step { ...props } label={ t('steps.internship-proposal.header') } active={ true } completed={ sent } until={ deadlines.proposal } state={ <ProposalStatus /> }>
        <p>{ t('steps.internship-proposal.info') }</p>

        <Button to={ route("internship_proposal") } variant="contained" color="primary" component={ RouterLink }>
            { t('steps.internship-proposal.form') }
        </Button>
    </Step>;
}

export const MainPage = () => {
    const { t } = useTranslation();

    const student = useSelector<AppState, Student | null>(state => state.student);
    const deadlines = useSelector<AppState, Deadlines>(state => getEditionDeadlines(state.edition as Edition)); // edition cannot be null at this point

    const missingStudentData = useMemo(() => student ? getMissingStudentData(student) : [], [student]);

    return <Page my={ 6 }>
        <Container>
            <Typography variant="h2">{ t("sections.my-internship.header") }</Typography>
            <Stepper orientation="vertical" nonLinear>
                <Step label={ t('steps.personal-data.header') } completed={ missingStudentData.length === 0 } until={ deadlines.personalData }>
                    { missingStudentData.length > 0 && <>
                        <p>{ t('steps.personal-data.info') }</p>

                        <ul>
                            { missingStudentData.map(field => <li key={ field }>{ t(`student.${ field }`) }</li>) }
                        </ul>

                        <Button to={ route("internship_proposal") } variant="contained" color="primary" component={ RouterLink }>
                            { t('steps.personal-data.form') }
                        </Button>
                    </> }
                </Step>
                <ProposalStep />
                <Step label={ t('steps.plan.header') } active={ missingStudentData.length === 0 } until={ deadlines.proposal }>
                    <p>{ t('steps.plan.info') }</p>

                    <Actions>
                        <Button to={ route("internship_plan") } variant="contained" color="primary" component={ RouterLink }>
                            { t('steps.plan.submit') }
                        </Button>
                        <Button href="https://eti.pg.edu.pl/documents/611675/100028367/indywidualny%20program%20praktyk" startIcon={ <DescriptionIcon/> }>
                            { t('steps.plan.template') }
                        </Button>
                    </Actions>
                </Step>
                <Step label={ t('steps.insurance.header') }/>
                <Step label={ t('steps.report.header') } until={ deadlines.report }/>
                <Step label={ t('steps.grade.header') }/>
            </Stepper>
        </Container>
    </Page>
}
